//-----------------------------------------------------------------------------
// Copyright  2000 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	avlmap
// homepage	http://phil.ipal.org/freeware/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map.h
//
// purpose	Define resources for the Data Mapping library package.
//-----------------------------------------------------------------------------
#ifndef __MAP_H__
#define __MAP_H__

//-----------------------------------------------------------------------------
// includes
//-----------------------------------------------------------------------------
#include <ctype.h>
#include <avl.h>

//-----------------------------------------------------------------------------
// structs
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// struct	map_root
// type		MAP
//
// purpose	This structure is the anchor point for a data mapping.
//		The pointer to it is the mapping handle.
//-----------------------------------------------------------------------------
struct map_root {
    int ( *	cmp_func	)()	;
    int		options		;
    int		key_type	;
    avl_root	avl_tree	;
};
typedef struct map_root		MAP_T	;
typedef struct map_root *	MAP	;

#define MAP_OPT_LOWER	1
#define MAP_OPT_UPPER	2
#define MAP_OPT_DIGITS	4
#define MAP_OPT_SPACES	8
#define MAP_OPT_PATHS	16


//-----------------------------------------------------------------------------
// macros
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Define a macro which given a pointer to the avl_root structure, yields a
// pointer to the map_root structure that contains it.
//-----------------------------------------------------------------------------
#define map_from_avl(p) ((MAP)(((char*)(p))-((size_t)(&(((MAP)0)->avl_tree)))))

//-----------------------------------------------------------------------------
// Define a macro that yields the function pointer to the alternate comparison
// function, given a pointer to the avl_tree member, which the avl layer does
// as the 3rd argument in the avl comparison function.
//-----------------------------------------------------------------------------
#if 0
#define map_cmp_func(r) (((MAP)(((char*)(r))-((size_t)(&(((MAP)0)->avl_tree)))))->cmp_func)
#else
#define map_cmp_func(p) (map_from_avl((p))->cmp_func)
#endif

//-----------------------------------------------------------------------------
// Define macros to implement string functions as array calls.  The array
// functions have a length argument.  The string functions do not have a
// length argument and use a value of zero to terminate a string.  These
// zero terminated strings are stored in the mapping as an array, with a
// length, so any zero terminated string given for either key or data, may
// be accessed as an array.  Array data may have embedded zero values, but
// otherwise arrays and strings share the same key space in mappings.
//-----------------------------------------------------------------------------
#define map_cs_new()			map_ca_new()
#define map_cs_cmp_func(m,f)		map_ca_cmp_func((m),(f))
#define map_cs_cmp_options(m,s,o)	map_ca_cmp_options((m),(s),(o))
#define map_cs_cmp_lower(m,s)		map_ca_cmp_lower((m),(s))
#define map_cs_cmp_upper(m,s)		map_ca_cmp_upper((m),(s))
#define map_cs_cmp_digits(m,s)		map_ca_cmp_digits((m),(s))
#define map_cs_cmp_spaces(m,s)		map_ca_cmp_spaces((m),(s))
#define map_cs_cmp_paths(m,s)		map_ca_cmp_paths((m),(s))
#define map_cs_find(m,k,d)		map_ca_find((m),(k),~0,(d))
#define map_cs_find_exact(m,k)		map_ca_find_exact((m),(k),~0)
#define map_cs_find_exact_lo(m,k)	map_ca_find_exact_lo((m),(k),~0)
#define map_cs_find_lo_or_near(m,k)	map_ca_find_lo_or_near((m),(k),~0)
#define map_cs_find_exact_hi(m,k)	map_ca_find_exact_hi((m),(k),~0)
#define map_cs_find_hi_or_near(m,k)	map_ca_find_hi_or_near((m),(k),~0)
#define map_cs_insert(m,k)		map_ca_insert((m),(k),~0)
#define map_cs_insert_dup(m,k)		map_ca_insert_dup((m),(k),~0)
#define map_cs_selected(m)		map_ca_selected((m))
#define map_cs_key_ptr(m)		map_ca_key_ptr((m))
#define map_cs_key_dup(m)		map_ca_key_dup((m))
#define map_cs_key_copy(m,kp,l)		map_ca_key_copy((m),(kp),(l))
#define map_cs_key_len(m)		map_ca_key_len((m))
#define map_cs_store(m,d)		map_ca_store((m),(d),~0)
#define map_cs_store_share(m,d)		map_ca_store_share((m),(d),~0)
#define map_cs_fetch_ptr(m)		map_ca_fetch_ptr((m))
#define map_cs_fetch_dup(m)		map_ca_fetch_dup((m))
#define map_cs_fetch_copy(m,dp,l)	map_ca_fetch_copy((m),(dp),(l))
#define map_cs_fetch_len(m)		map_ca_fetch_len((m))

#define map_str_new()			map_ca_new()
#define map_str_cmp_func(m,f)		map_ca_cmp_func((m),(f))
#define map_str_cmp_options(m,s,o)	map_ca_cmp_options((m),(s),(o))
#define map_str_cmp_lower(m,s)		map_ca_cmp_lower((m),(s))
#define map_str_cmp_upper(m,s)		map_ca_cmp_upper((m),(s))
#define map_str_cmp_digits(m,s)		map_ca_cmp_digits((m),(s))
#define map_str_cmp_spaces(m,s)		map_ca_cmp_spaces((m),(s))
#define map_str_cmp_paths(m,s)		map_ca_cmp_paths((m),(s))
#define map_str_find(m,k,d)		map_ca_find((m),(k),~0,(d))
#define map_str_find_exact(m,k)		map_ca_find_exact((m),(k),~0)
#define map_str_find_exact_lo(m,k)	map_ca_find_exact_lo((m),(k),~0)
#define map_str_find_lo_or_near(m,k)	map_ca_find_lo_or_near((m),(k),~0)
#define map_str_find_exact_hi(m,k)	map_ca_find_exact_hi((m),(k),~0)
#define map_str_find_hi_or_near(m,k)	map_ca_find_hi_or_near((m),(k),~0)
#define map_str_insert(m,k)		map_ca_insert((m),(k),~0)
#define map_str_insert_dup(m,k)		map_ca_insert_dup((m),(k),~0)
#define map_str_selected(m)		map_ca_selected((m))
#define map_str_key_ptr(m)		map_ca_key_ptr((m))
#define map_str_key_dup(m)		map_ca_key_dup((m))
#define map_str_key_copy(m,kp,l)	map_ca_key_copy((m),(kp),(l))
#define map_str_key_len(m)		map_ca_key_len((m))
#define map_str_store(m,d)		map_ca_store((m),(d),~0)
#define map_str_store_share(m,d)	map_ca_store_share((m),(d),~0)
#define map_str_fetch_ptr(m)		map_ca_fetch_ptr((m))
#define map_str_fetch_dup(m)		map_ca_fetch_dup((m))
#define map_str_fetch_copy(m,dp,l)	map_ca_fetch_copy((m),(dp),(l))
#define map_str_fetch_len(m)		map_ca_fetch_len((m))

#define map_scs_new()			map_sca_new()
#define map_scs_cmp_func(m,f)		map_sca_cmp_func((m),(f))
#define map_scs_cmp_options(m,s,o)	map_sca_cmp_options((m),(s),(o))
#define map_scs_cmp_lower(m,s)		map_sca_cmp_lower((m),(s))
#define map_scs_cmp_upper(m,s)		map_sca_cmp_upper((m),(s))
#define map_scs_cmp_digits(m,s)		map_sca_cmp_digits((m),(s))
#define map_scs_cmp_spaces(m,s)		map_sca_cmp_spaces((m),(s))
#define map_scs_cmp_paths(m,s)		map_sca_cmp_paths((m),(s))
#define map_scs_find(m,k,d)		map_sca_find((m),(k),~0,(d))
#define map_scs_find_exact(m,k)		map_sca_find_exact((m),(k),~0)
#define map_scs_find_exact_lo(m,k)	map_sca_find_exact_lo((m),(k),~0)
#define map_scs_find_lo_or_near(m,k)	map_sca_find_lo_or_near((m),(k),~0)
#define map_scs_find_exact_hi(m,k)	map_sca_find_exact_hi((m),(k),~0)
#define map_scs_find_hi_or_near(m,k)	map_sca_find_hi_or_near((m),(k),~0)
#define map_scs_insert(m,k)		map_sca_insert((m),(k),~0)
#define map_scs_insert_dup(m,k)		map_sca_insert_dup((m),(k),~0)
#define map_scs_selected(m)		map_sca_selected((m))
#define map_scs_key_ptr(m)		map_sca_key_ptr((m))
#define map_scs_key_dup(m)		map_sca_key_dup((m))
#define map_scs_key_copy(m,kp,l)	map_sca_key_copy((m),(kp),(l))
#define map_scs_key_len(m)		map_sca_key_len((m))
#define map_scs_store(m,d)		map_sca_store((m),(d),~0)
#define map_scs_store_share(m,d)	map_sca_store_share((m),(d),~0)
#define map_scs_fetch_ptr(m)		map_sca_fetch_ptr((m))
#define map_scs_fetch_dup(m)		map_sca_fetch_dup((m))
#define map_scs_fetch_copy(m,dp,l)	map_sca_fetch_copy((m),(dp),(l))
#define map_scs_fetch_len(m)		map_sca_fetch_len((m))

#define map_ucs_new()			map_uca_new()
#define map_ucs_cmp_func(m,f)		map_uca_cmp_func((m),(f))
#define map_ucs_cmp_options(m,s,o)	map_uca_cmp_options((m),(s),(o))
#define map_ucs_cmp_lower(m,s)		map_uca_cmp_lower((m),(s))
#define map_ucs_cmp_upper(m,s)		map_uca_cmp_upper((m),(s))
#define map_ucs_cmp_digits(m,s)		map_uca_cmp_digits((m),(s))
#define map_ucs_cmp_spaces(m,s)		map_uca_cmp_spaces((m),(s))
#define map_ucs_cmp_paths(m,s)		map_uca_cmp_paths((m),(s))
#define map_ucs_find(m,k,d)		map_uca_find((m),(k),~0,(d))
#define map_ucs_find_exact(m,k)		map_uca_find_exact((m),(k),~0)
#define map_ucs_find_exact_lo(m,k)	map_uca_find_exact_lo((m),(k),~0)
#define map_ucs_find_lo_or_near(m,k)	map_uca_find_lo_or_near((m),(k),~0)
#define map_ucs_find_exact_hi(m,k)	map_uca_find_exact_hi((m),(k),~0)
#define map_ucs_find_hi_or_near(m,k)	map_uca_find_hi_or_near((m),(k),~0)
#define map_ucs_insert(m,k)		map_uca_insert((m),(k),~0)
#define map_ucs_insert_dup(m,k)		map_uca_insert_dup((m),(k),~0)
#define map_ucs_selected(m)		map_uca_selected((m))
#define map_ucs_key_ptr(m)		map_uca_key_ptr((m))
#define map_ucs_key_dup(m)		map_uca_key_dup((m))
#define map_ucs_key_copy(m,kp,l)	map_uca_key_copy((m),(kp),(l))
#define map_ucs_key_len(m)		map_uca_key_len((m))
#define map_ucs_store(m,d)		map_uca_store((m),(d),~0)
#define map_ucs_store_share(m,d)	map_uca_store_share((m),(d),~0)
#define map_ucs_fetch_ptr(m)		map_uca_fetch_ptr((m))
#define map_ucs_fetch_dup(m)		map_uca_fetch_dup((m))
#define map_ucs_fetch_copy(m,dp,l)	map_uca_fetch_copy((m),(dp),(l))
#define map_ucs_fetch_len(m)		map_uca_fetch_len((m))

#define map_sss_new()			map_ssa_new()
#define map_sss_cmp_func(m,f)		map_ssa_cmp_func((m),(f))
#define map_sss_find(m,k,d)		map_ssa_find((m),(k),~0,(d))
#define map_sss_find_exact(m,k)		map_ssa_find_exact((m),(k),~0)
#define map_sss_find_exact_lo(m,k)	map_ssa_find_exact_lo((m),(k),~0)
#define map_sss_find_lo_or_near(m,k)	map_ssa_find_lo_or_near((m),(k),~0)
#define map_sss_find_exact_hi(m,k)	map_ssa_find_exact_hi((m),(k),~0)
#define map_sss_find_hi_or_near(m,k)	map_ssa_find_hi_or_near((m),(k),~0)
#define map_sss_insert(m,k)		map_ssa_insert((m),(k),~0)
#define map_sss_insert_dup(m,k)		map_ssa_insert_dup((m),(k),~0)
#define map_sss_selected(m)		map_ssa_selected((m))
#define map_sss_key_ptr(m)		map_ssa_key_ptr((m))
#define map_sss_key_dup(m)		map_ssa_key_dup((m))
#define map_sss_key_copy(m,kp,l)	map_ssa_key_copy((m),(kp),(l))
#define map_sss_key_len(m)		map_ssa_key_len((m))
#define map_sss_store(m,d)		map_ssa_store((m),(d),~0)
#define map_sss_store_share(m,d)	map_ssa_store_share((m),(d),~0)
#define map_sss_fetch_ptr(m)		map_ssa_fetch_ptr((m))
#define map_sss_fetch_dup(m)		map_ssa_fetch_dup((m))
#define map_sss_fetch_copy(m,dp,l)	map_ssa_fetch_copy((m),(dp),(l))
#define map_sss_fetch_len(m)		map_ssa_fetch_len((m))

#define map_uss_new()			map_usa_new()
#define map_uss_cmp_func(m,f)		map_usa_cmp_func((m),(f))
#define map_uss_find(m,k,d)		map_usa_find((m),(k),~0,(d))
#define map_uss_find_exact(m,k)		map_usa_find_exact((m),(k),~0)
#define map_uss_find_exact_lo(m,k)	map_usa_find_exact_lo((m),(k),~0)
#define map_uss_find_lo_or_near(m,k)	map_usa_find_lo_or_near((m),(k),~0)
#define map_uss_find_exact_hi(m,k)	map_usa_find_exact_hi((m),(k),~0)
#define map_uss_find_hi_or_near(m,k)	map_usa_find_hi_or_near((m),(k),~0)
#define map_uss_insert(m,k)		map_usa_insert((m),(k),~0)
#define map_uss_insert_dup(m,k)		map_usa_insert_dup((m),(k),~0)
#define map_uss_selected(m)		map_usa_selected((m))
#define map_uss_key_ptr(m)		map_usa_key_ptr((m))
#define map_uss_key_dup(m)		map_usa_key_dup((m))
#define map_uss_key_copy(m,kp,l)	map_usa_key_copy((m),(kp),(l))
#define map_uss_key_len(m)		map_usa_key_len((m))
#define map_uss_store(m,d)		map_usa_store((m),(d),~0)
#define map_uss_store_share(m,d)	map_usa_store_share((m),(d),~0)
#define map_uss_fetch_ptr(m)		map_usa_fetch_ptr((m))
#define map_uss_fetch_dup(m)		map_usa_fetch_dup((m))
#define map_uss_fetch_copy(m,dp,l)	map_usa_fetch_copy((m),(dp),(l))
#define map_uss_fetch_len(m)		map_usa_fetch_len((m))

#define map_sis_new()			map_sia_new()
#define map_sis_cmp_func(m,f)		map_sia_cmp_func((m),(f))
#define map_sis_find(m,k,d)		map_sia_find((m),(k),~0,(d))
#define map_sis_find_exact(m,k)		map_sia_find_exact((m),(k),~0)
#define map_sis_find_exact_lo(m,k)	map_sia_find_exact_lo((m),(k),~0)
#define map_sis_find_lo_or_near(m,k)	map_sia_find_lo_or_near((m),(k),~0)
#define map_sis_find_exact_hi(m,k)	map_sia_find_exact_hi((m),(k),~0)
#define map_sis_find_hi_or_near(m,k)	map_sia_find_hi_or_near((m),(k),~0)
#define map_sis_insert(m,k)		map_sia_insert((m),(k),~0)
#define map_sis_insert_dup(m,k)		map_sia_insert_dup((m),(k),~0)
#define map_sis_selected(m)		map_sia_selected((m))
#define map_sis_key_ptr(m)		map_sia_key_ptr((m))
#define map_sis_key_dup(m)		map_sia_key_dup((m))
#define map_sis_key_copy(m,kp,l)	map_sia_key_copy((m),(kp),(l))
#define map_sis_key_len(m)		map_sia_key_len((m))
#define map_sis_store(m,d)		map_sia_store((m),(d),~0)
#define map_sis_store_share(m,d)	map_sia_store_share((m),(d),~0)
#define map_sis_fetch_ptr(m)		map_sia_fetch_ptr((m))
#define map_sis_fetch_dup(m)		map_sia_fetch_dup((m))
#define map_sis_fetch_copy(m,dp,l)	map_sia_fetch_copy((m),(dp),(l))
#define map_sis_fetch_len(m)		map_sia_fetch_len((m))

#define map_uis_new()			map_uia_new()
#define map_uis_cmp_func(m,f)		map_uia_cmp_func((m),(f))
#define map_uis_find(m,k,d)		map_uia_find((m),(k),~0,(d))
#define map_uis_find_exact(m,k)		map_uia_find_exact((m),(k),~0)
#define map_uis_find_exact_lo(m,k)	map_uia_find_exact_lo((m),(k),~0)
#define map_uis_find_lo_or_near(m,k)	map_uia_find_lo_or_near((m),(k),~0)
#define map_uis_find_exact_hi(m,k)	map_uia_find_exact_hi((m),(k),~0)
#define map_uis_find_hi_or_near(m,k)	map_uia_find_hi_or_near((m),(k),~0)
#define map_uis_insert(m,k)		map_uia_insert((m),(k),~0)
#define map_uis_insert_dup(m,k)		map_uia_insert_dup((m),(k),~0)
#define map_uis_selected(m)		map_uia_selected((m))
#define map_uis_key_ptr(m)		map_uia_key_ptr((m))
#define map_uis_key_dup(m)		map_uia_key_dup((m))
#define map_uis_key_copy(m,kp,l)	map_uia_key_copy((m),(kp),(l))
#define map_uis_key_len(m)		map_uia_key_len((m))
#define map_uis_store(m,d)		map_uia_store((m),(d),~0)
#define map_uis_store_share(m,d)	map_uia_store_share((m),(d),~0)
#define map_uis_fetch_ptr(m)		map_uia_fetch_ptr((m))
#define map_uis_fetch_dup(m)		map_uia_fetch_dup((m))
#define map_uis_fetch_copy(m,dp,l)	map_uia_fetch_copy((m),(dp),(l))
#define map_uis_fetch_len(m)		map_uia_fetch_len((m))

#define map_sls_new()			map_sla_new()
#define map_sls_cmp_func(m,f)		map_sla_cmp_func((m),(f))
#define map_sls_find(m,k,d)		map_sla_find((m),(k),~0,(d))
#define map_sls_find_exact(m,k)		map_sla_find_exact((m),(k),~0)
#define map_sls_find_exact_lo(m,k)	map_sla_find_exact_lo((m),(k),~0)
#define map_sls_find_lo_or_near(m,k)	map_sla_find_lo_or_near((m),(k),~0)
#define map_sls_find_exact_hi(m,k)	map_sla_find_exact_hi((m),(k),~0)
#define map_sls_find_hi_or_near(m,k)	map_sla_find_hi_or_near((m),(k),~0)
#define map_sls_insert(m,k)		map_sla_insert((m),(k),~0)
#define map_sls_insert_dup(m,k)		map_sla_insert_dup((m),(k),~0)
#define map_sls_selected(m)		map_sla_selected((m))
#define map_sls_key_ptr(m)		map_sla_key_ptr((m))
#define map_sls_key_dup(m)		map_sla_key_dup((m))
#define map_sls_key_copy(m,kp,l)	map_sla_key_copy((m),(kp),(l))
#define map_sls_key_len(m)		map_sla_key_len((m))
#define map_sls_store(m,d)		map_sla_store((m),(d),~0)
#define map_sls_store_share(m,d)	map_sla_store_share((m),(d),~0)
#define map_sls_fetch_ptr(m)		map_sla_fetch_ptr((m))
#define map_sls_fetch_dup(m)		map_sla_fetch_dup((m))
#define map_sls_fetch_copy(m,dp,l)	map_sla_fetch_copy((m),(dp),(l))
#define map_sls_fetch_len(m)		map_sla_fetch_len((m))

#define map_uls_new()			map_ula_new()
#define map_uls_cmp_func(m,f)		map_ula_cmp_func((m),(f))
#define map_uls_find(m,k,d)		map_ula_find((m),(k),~0,(d))
#define map_uls_find_exact(m,k)		map_ula_find_exact((m),(k),~0)
#define map_uls_find_exact_lo(m,k)	map_ula_find_exact_lo((m),(k),~0)
#define map_uls_find_lo_or_near(m,k)	map_ula_find_lo_or_near((m),(k),~0)
#define map_uls_find_exact_hi(m,k)	map_ula_find_exact_hi((m),(k),~0)
#define map_uls_find_hi_or_near(m,k)	map_ula_find_hi_or_near((m),(k),~0)
#define map_uls_insert(m,k)		map_ula_insert((m),(k),~0)
#define map_uls_insert_dup(m,k)		map_ula_insert_dup((m),(k),~0)
#define map_uls_selected(m)		map_ula_selected((m))
#define map_uls_key_ptr(m)		map_ula_key_ptr((m))
#define map_uls_key_dup(m)		map_ula_key_dup((m))
#define map_uls_key_copy(m,kp,l)	map_ula_key_copy((m),(kp),(l))
#define map_uls_key_len(m)		map_ula_key_len((m))
#define map_uls_store(m,d)		map_ula_store((m),(d),~0)
#define map_uls_store_share(m,d)	map_ula_store_share((m),(d),~0)
#define map_uls_fetch_ptr(m)		map_ula_fetch_ptr((m))
#define map_uls_fetch_dup(m)		map_ula_fetch_dup((m))
#define map_uls_fetch_copy(m,dp,l)	map_ula_fetch_copy((m),(dp),(l))
#define map_uls_fetch_len(m)		map_ula_fetch_len((m))

#define map_slls_new()			map_slla_new()
#define map_slls_cmp_func(m,f)		map_slla_cmp_func((m),(f))
#define map_slls_find(m,k,d)		map_slla_find((m),(k),~0,(d))
#define map_slls_find_exact(m,k)	map_slla_find_exact((m),(k),~0)
#define map_slls_find_exact_lo(m,k)	map_slla_find_exact_lo((m),(k),~0)
#define map_slls_find_lo_or_near(m,k)	map_slla_find_lo_or_near((m),(k),~0)
#define map_slls_find_exact_hi(m,k)	map_slla_find_exact_hi((m),(k),~0)
#define map_slls_find_hi_or_near(m,k)	map_slla_find_hi_or_near((m),(k),~0)
#define map_slls_insert(m,k)		map_slla_insert((m),(k),~0)
#define map_slls_insert_dup(m,k)	map_slla_insert_dup((m),(k),~0)
#define map_slls_selected(m)		map_slla_selected((m))
#define map_slls_key_ptr(m)		map_slla_key_ptr((m))
#define map_slls_key_dup(m)		map_slla_key_dup((m))
#define map_slls_key_copy(m,kp,l)	map_slla_key_copy((m),(kp),(l))
#define map_slls_key_len(m)		map_slla_key_len((m))
#define map_slls_store(m,d)		map_slla_store((m),(d),~0)
#define map_slls_store_share(m,d)	map_slla_store_share((m),(d),~0)
#define map_slls_fetch_ptr(m)		map_slla_fetch_ptr((m))
#define map_slls_fetch_dup(m)		map_slla_fetch_dup((m))
#define map_slls_fetch_copy(m,dp,l)	map_slla_fetch_copy((m),(dp),(l))
#define map_slls_fetch_len(m)		map_slla_fetch_len((m))

#define map_ulls_new()			map_ulla_new()
#define map_ulls_cmp_func(m,f)		map_ulla_cmp_func((m),(f))
#define map_ulls_find(m,k,d)		map_ulla_find((m),(k),~0,(d))
#define map_ulls_find_exact(m,k)	map_ulla_find_exact((m),(k),~0)
#define map_ulls_find_exact_lo(m,k)	map_ulla_find_exact_lo((m),(k),~0)
#define map_ulls_find_lo_or_near(m,k)	map_ulla_find_lo_or_near((m),(k),~0)
#define map_ulls_find_exact_hi(m,k)	map_ulla_find_exact_hi((m),(k),~0)
#define map_ulls_find_hi_or_near(m,k)	map_ulla_find_hi_or_near((m),(k),~0)
#define map_ulls_insert(m,k)		map_ulla_insert((m),(k),~0)
#define map_ulls_insert_dup(m,k)	map_ulla_insert_dup((m),(k),~0)
#define map_ulls_selected(m)		map_ulla_selected((m))
#define map_ulls_key_ptr(m)		map_ulla_key_ptr((m))
#define map_ulls_key_dup(m)		map_ulla_key_dup((m))
#define map_ulls_key_copy(m,kp,l)	map_ulla_key_copy((m),(kp),(l))
#define map_ulls_key_len(m)		map_ulla_key_len((m))
#define map_ulls_store(m,d)		map_ulla_store((m),(d),~0)
#define map_ulls_store_share(m,d)	map_ulla_store_share((m),(d),~0)
#define map_ulls_fetch_ptr(m)		map_ulla_fetch_ptr((m))
#define map_ulls_fetch_dup(m)		map_ulla_fetch_dup((m))
#define map_ulls_fetch_copy(m,dp,l)	map_ulla_fetch_copy((m),(dp),(l))
#define map_ulls_fetch_len(m)		map_ulla_fetch_len((m))


//-- Define various macros to describe type properties.
#define MAP_DATA_SIGNED		0x00000100
#define MAP_DATA_UNSIGNED	0x00000200

#define MAP_DATA_INTEGER	0x00000400
#define MAP_DATA_FLOAT		0x00000800
#define MAP_DATA_OTHER		0x00001000

#define MAP_DATA_SCALAR		0x00002000
#define MAP_DATA_STRING		0x00004000

#define MAP_DATA_DOUBLE		(sizeof(double))
#define MAP_DATA_LONG_DOUBLE	(sizeof(long double))

#define MAP_DATA_CHAR		(sizeof(char))
#define MAP_DATA_SHORT		(sizeof(short))
#define MAP_DATA_INT		(sizeof(int))
#define MAP_DATA_LONG		(sizeof(long))
#define MAP_DATA_LONG_LONG	(sizeof(long long))

#define MAP_DATA_POINTER	(sizeof(void *))

//-- Define macros that can be used to isolate properties of a type.
#define map_type_is_signed(t)		\
	(((t)&((MAP_DATA_SIGNED)|(MAP_DATA_UNSIGNED)))==(MAP_DATA_SIGNED))
#define map_type_is_unsigned(t)		\
	(((t)&((MAP_DATA_SIGNED)|(MAP_DATA_UNSIGNED)))==(MAP_DATA_UNSIGNED))

#define map_type_is_integer(t)		\
	(((t)&((MAP_DATA_INTEGER)|(MAP_DATA_FLOAT)|(MAP_DATA_OTHER)))==(MAP_DATA_INTEGER))
#define map_type_is_float(t)		\
	(((t)&((MAP_DATA_INTEGER)|(MAP_DATA_FLOAT)|(MAP_DATA_OTHER)))==(MAP_DATA_FLOAT))

#define map_type_is_scalar(t)		\
	(((t)&((MAP_DATA_SCALAR)|(MAP_DATA_STRING)))==(MAP_DATA_SCALAR))
#define map_type_is_string(t)		\
	(((t)&((MAP_DATA_SCALAR)|(MAP_DATA_STRING)))==(MAP_DATA_STRING))

#define map_type_sizeof(t)	((t)&(0x000000ff))

//-- Define macros for each individual type.
#define MAP_DATA_C	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_CHAR
#define MAP_DATA_SC	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UC	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SS	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_US	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SI	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UI	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_UL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG

#define MAP_DATA_D	MAP_DATA_SCALAR+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LD	MAP_DATA_SCALAR+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

#define MAP_DATA_PTR	MAP_DATA_SCALAR+MAP_DATA_OTHER+MAP_DATA_POINTER

#define MAP_DATA_CA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_CHAR
#define MAP_DATA_SCA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UCA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SSA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_USA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SIA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UIA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SLA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_ULA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLLA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULLA	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG

#define MAP_DATA_DA	MAP_DATA_STRING+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LDA	MAP_DATA_STRING+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

#define MAP_DATA_CS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_CHAR
#define MAP_DATA_SCS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UCS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SSS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_USS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SIS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UIS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SLS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_ULS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLLS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULLS	MAP_DATA_STRING+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG

#define MAP_DATA_DS	MAP_DATA_STRING+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LDS	MAP_DATA_STRING+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

#define MAP_DATA_STR	MAP_DATA_CS

//-----------------------------------------------------------------------------
// macro	map_loop_forward
//
// purpose	Start a program loop which steps forward over all the
//		elements in a mapping, starting at the beginning.
//
// arguments	1 (MAP) which mapping
//
// usage	map_loop_forward( map ) { /* body of loop */ }
//
//		In the body of the loop, place the appropriate call to get
//		the data type and data that for that loop iteratiin.
//-----------------------------------------------------------------------------
#define map_loop_forward(m) for(map_unselect((m));map_forward((m))>=0;)

//-----------------------------------------------------------------------------
// macro	map_loop_reverse
//
// purpose	Start a program loop which steps in reverse over all the
//		elements in a mapping, starting at the end.
//
// arguments	1 (MAP) which mapping
//
// usage	map_loop_reverse( map ) { /* body of loop */ }
//
//		In the body of the loop, place the appropriate call to get
//		the data type and data that for that loop iteratiin.
//-----------------------------------------------------------------------------
#define map_loop_reverse(m) for(map_unselect((m));map_reverse((m))>=0;)


//-----------------------------------------------------------------------------
// functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// functions	map_ca_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ca_new
#define map_xx_cmp_func		map_ca_cmp_func
#define map_xx_cmp_options	map_ca_cmp_options
#define map_xx_cmp_lower	map_ca_cmp_lower
#define map_xx_cmp_upper	map_ca_cmp_upper
#define map_xx_cmp_digits	map_ca_cmp_digits
#define map_xx_cmp_spaces	map_ca_cmp_spaces
#define map_xx_cmp_paths	map_ca_cmp_paths
#define map_xx_find		map_ca_find
#define map_xx_find_exact	map_ca_find_exact
#define map_xx_find_exact_lo	map_ca_find_exact_lo
#define map_xx_find_lo_or_near	map_ca_find_lo_or_near
#define map_xx_find_exact_hi	map_ca_find_exact_hi
#define map_xx_find_hi_or_near	map_ca_find_hi_or_near
#define map_xx_insert		map_ca_insert
#define map_xx_insert_dup	map_ca_insert_dup
#define map_xx_selected		map_ca_selected
#define map_xx_key_ptr		map_ca_key_ptr
#define map_xx_key_dup		map_ca_key_dup
#define map_xx_key_copy		map_ca_key_copy
#define map_xx_key_len		map_ca_key_len
#define map_xx_store		map_ca_store
#define map_xx_store_share	map_ca_store_share
#define map_xx_fetch_ptr	map_ca_fetch_ptr
#define map_xx_fetch_dup	map_ca_fetch_dup
#define map_xx_fetch_copy	map_ca_fetch_copy
#define map_xx_fetch_len	map_ca_fetch_len
#define map_type		char

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_cmp_options
#undef map_xx_cmp_lower
#undef map_xx_cmp_upper
#undef map_xx_cmp_digits
#undef map_xx_cmp_spaces
#undef map_xx_cmp_paths
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_sca_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_sca_new
#define map_xx_cmp_func		map_sca_cmp_func
#define map_xx_cmp_options	map_sca_cmp_options
#define map_xx_cmp_lower	map_sca_cmp_lower
#define map_xx_cmp_upper	map_sca_cmp_upper
#define map_xx_cmp_digits	map_sca_cmp_digits
#define map_xx_cmp_spaces	map_sca_cmp_spaces
#define map_xx_cmp_paths	map_sca_cmp_paths
#define map_xx_find		map_sca_find
#define map_xx_find_exact	map_sca_find_exact
#define map_xx_find_exact_lo	map_sca_find_exact_lo
#define map_xx_find_lo_or_near	map_sca_find_lo_or_near
#define map_xx_find_exact_hi	map_sca_find_exact_hi
#define map_xx_find_hi_or_near	map_sca_find_hi_or_near
#define map_xx_insert		map_sca_insert
#define map_xx_insert_dup	map_sca_insert_dup
#define map_xx_selected		map_sca_selected
#define map_xx_key_ptr		map_sca_key_ptr
#define map_xx_key_dup		map_sca_key_dup
#define map_xx_key_copy		map_sca_key_copy
#define map_xx_key_len		map_sca_key_len
#define map_xx_store		map_sca_store
#define map_xx_store_share	map_sca_store_share
#define map_xx_fetch_ptr	map_sca_fetch_ptr
#define map_xx_fetch_dup	map_sca_fetch_dup
#define map_xx_fetch_copy	map_sca_fetch_copy
#define map_xx_fetch_len	map_sca_fetch_len
#define map_type		signed char

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_cmp_options
#undef map_xx_cmp_lower
#undef map_xx_cmp_upper
#undef map_xx_cmp_digits
#undef map_xx_cmp_spaces
#undef map_xx_cmp_paths
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_uca_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_uca_new
#define map_xx_cmp_func		map_uca_cmp_func
#define map_xx_cmp_options	map_uca_cmp_options
#define map_xx_cmp_lower	map_uca_cmp_lower
#define map_xx_cmp_upper	map_uca_cmp_upper
#define map_xx_cmp_digits	map_uca_cmp_digits
#define map_xx_cmp_spaces	map_uca_cmp_spaces
#define map_xx_cmp_paths	map_uca_cmp_paths
#define map_xx_find		map_uca_find
#define map_xx_find_exact	map_uca_find_exact
#define map_xx_find_exact_lo	map_uca_find_exact_lo
#define map_xx_find_lo_or_near	map_uca_find_lo_or_near
#define map_xx_find_exact_hi	map_uca_find_exact_hi
#define map_xx_find_hi_or_near	map_uca_find_hi_or_near
#define map_xx_insert		map_uca_insert
#define map_xx_insert_dup	map_uca_insert_dup
#define map_xx_selected		map_uca_selected
#define map_xx_key_ptr		map_uca_key_ptr
#define map_xx_key_dup		map_uca_key_dup
#define map_xx_key_copy		map_uca_key_copy
#define map_xx_key_len		map_uca_key_len
#define map_xx_store		map_uca_store
#define map_xx_store_share	map_uca_store_share
#define map_xx_fetch_ptr	map_uca_fetch_ptr
#define map_xx_fetch_dup	map_uca_fetch_dup
#define map_xx_fetch_copy	map_uca_fetch_copy
#define map_xx_fetch_len	map_uca_fetch_len
#define map_type		unsigned char

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_cmp_options
#undef map_xx_cmp_lower
#undef map_xx_cmp_upper
#undef map_xx_cmp_digits
#undef map_xx_cmp_spaces
#undef map_xx_cmp_paths
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ssa_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ssa_new
#define map_xx_cmp_func		map_ssa_cmp_func
#define map_xx_find		map_ssa_find
#define map_xx_find_exact	map_ssa_find_exact
#define map_xx_find_exact_lo	map_ssa_find_exact_lo
#define map_xx_find_lo_or_near	map_ssa_find_lo_or_near
#define map_xx_find_exact_hi	map_ssa_find_exact_hi
#define map_xx_find_hi_or_near	map_ssa_find_hi_or_near
#define map_xx_insert		map_ssa_insert
#define map_xx_insert_dup	map_ssa_insert_dup
#define map_xx_selected		map_ssa_selected
#define map_xx_key_ptr		map_ssa_key_ptr
#define map_xx_key_dup		map_ssa_key_dup
#define map_xx_key_copy		map_ssa_key_copy
#define map_xx_key_len		map_ssa_key_len
#define map_xx_store		map_ssa_store
#define map_xx_store_share	map_ssa_store_share
#define map_xx_fetch_ptr	map_ssa_fetch_ptr
#define map_xx_fetch_dup	map_ssa_fetch_dup
#define map_xx_fetch_copy	map_ssa_fetch_copy
#define map_xx_fetch_len	map_ssa_fetch_len
#define map_type		signed short

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_usa_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_usa_new
#define map_xx_cmp_func		map_usa_cmp_func
#define map_xx_find		map_usa_find
#define map_xx_find_exact	map_usa_find_exact
#define map_xx_find_exact_lo	map_usa_find_exact_lo
#define map_xx_find_lo_or_near	map_usa_find_lo_or_near
#define map_xx_find_exact_hi	map_usa_find_exact_hi
#define map_xx_find_hi_or_near	map_usa_find_hi_or_near
#define map_xx_insert		map_usa_insert
#define map_xx_insert_dup	map_usa_insert_dup
#define map_xx_selected		map_usa_selected
#define map_xx_key_ptr		map_usa_key_ptr
#define map_xx_key_dup		map_usa_key_dup
#define map_xx_key_copy		map_usa_key_copy
#define map_xx_key_len		map_usa_key_len
#define map_xx_store		map_usa_store
#define map_xx_store_share	map_usa_store_share
#define map_xx_fetch_ptr	map_usa_fetch_ptr
#define map_xx_fetch_dup	map_usa_fetch_dup
#define map_xx_fetch_copy	map_usa_fetch_copy
#define map_xx_fetch_len	map_usa_fetch_len
#define map_type		unsigned short

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_sia_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_sia_new
#define map_xx_cmp_func		map_sia_cmp_func
#define map_xx_find		map_sia_find
#define map_xx_find_exact	map_sia_find_exact
#define map_xx_find_exact_lo	map_sia_find_exact_lo
#define map_xx_find_lo_or_near	map_sia_find_lo_or_near
#define map_xx_find_exact_hi	map_sia_find_exact_hi
#define map_xx_find_hi_or_near	map_sia_find_hi_or_near
#define map_xx_insert		map_sia_insert
#define map_xx_insert_dup	map_sia_insert_dup
#define map_xx_selected		map_sia_selected
#define map_xx_key_ptr		map_sia_key_ptr
#define map_xx_key_dup		map_sia_key_dup
#define map_xx_key_copy		map_sia_key_copy
#define map_xx_key_len		map_sia_key_len
#define map_xx_store		map_sia_store
#define map_xx_store_share	map_sia_store_share
#define map_xx_fetch_ptr	map_sia_fetch_ptr
#define map_xx_fetch_dup	map_sia_fetch_dup
#define map_xx_fetch_copy	map_sia_fetch_copy
#define map_xx_fetch_len	map_sia_fetch_len
#define map_type		signed int

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_uia_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_uia_new
#define map_xx_cmp_func		map_uia_cmp_func
#define map_xx_find		map_uia_find
#define map_xx_find_exact	map_uia_find_exact
#define map_xx_find_exact_lo	map_uia_find_exact_lo
#define map_xx_find_lo_or_near	map_uia_find_lo_or_near
#define map_xx_find_exact_hi	map_uia_find_exact_hi
#define map_xx_find_hi_or_near	map_uia_find_hi_or_near
#define map_xx_insert		map_uia_insert
#define map_xx_insert_dup	map_uia_insert_dup
#define map_xx_selected		map_uia_selected
#define map_xx_key_ptr		map_uia_key_ptr
#define map_xx_key_dup		map_uia_key_dup
#define map_xx_key_copy		map_uia_key_copy
#define map_xx_key_len		map_uia_key_len
#define map_xx_store		map_uia_store
#define map_xx_store_share	map_uia_store_share
#define map_xx_fetch_ptr	map_uia_fetch_ptr
#define map_xx_fetch_dup	map_uia_fetch_dup
#define map_xx_fetch_copy	map_uia_fetch_copy
#define map_xx_fetch_len	map_uia_fetch_len
#define map_type		unsigned int

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_sla_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_sla_new
#define map_xx_cmp_func		map_sla_cmp_func
#define map_xx_find		map_sla_find
#define map_xx_find_exact	map_sla_find_exact
#define map_xx_find_exact_lo	map_sla_find_exact_lo
#define map_xx_find_lo_or_near	map_sla_find_lo_or_near
#define map_xx_find_exact_hi	map_sla_find_exact_hi
#define map_xx_find_hi_or_near	map_sla_find_hi_or_near
#define map_xx_insert		map_sla_insert
#define map_xx_insert_dup	map_sla_insert_dup
#define map_xx_selected		map_sla_selected
#define map_xx_key_ptr		map_sla_key_ptr
#define map_xx_key_dup		map_sla_key_dup
#define map_xx_key_copy		map_sla_key_copy
#define map_xx_key_len		map_sla_key_len
#define map_xx_store		map_sla_store
#define map_xx_store_share	map_sla_store_share
#define map_xx_fetch_ptr	map_sla_fetch_ptr
#define map_xx_fetch_dup	map_sla_fetch_dup
#define map_xx_fetch_copy	map_sla_fetch_copy
#define map_xx_fetch_len	map_sla_fetch_len
#define map_type		signed long

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ula_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ula_new
#define map_xx_cmp_func		map_ula_cmp_func
#define map_xx_find		map_ula_find
#define map_xx_find_exact	map_ula_find_exact
#define map_xx_find_exact_lo	map_ula_find_exact_lo
#define map_xx_find_lo_or_near	map_ula_find_lo_or_near
#define map_xx_find_exact_hi	map_ula_find_exact_hi
#define map_xx_find_hi_or_near	map_ula_find_hi_or_near
#define map_xx_insert		map_ula_insert
#define map_xx_insert_dup	map_ula_insert_dup
#define map_xx_selected		map_ula_selected
#define map_xx_key_ptr		map_ula_key_ptr
#define map_xx_key_dup		map_ula_key_dup
#define map_xx_key_copy		map_ula_key_copy
#define map_xx_key_len		map_ula_key_len
#define map_xx_store		map_ula_store
#define map_xx_store_share	map_ula_store_share
#define map_xx_fetch_ptr	map_ula_fetch_ptr
#define map_xx_fetch_dup	map_ula_fetch_dup
#define map_xx_fetch_copy	map_ula_fetch_copy
#define map_xx_fetch_len	map_ula_fetch_len
#define map_type		unsigned long

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_slla_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_slla_new
#define map_xx_cmp_func		map_slla_cmp_func
#define map_xx_find		map_slla_find
#define map_xx_find_exact	map_slla_find_exact
#define map_xx_find_exact_lo	map_slla_find_exact_lo
#define map_xx_find_lo_or_near	map_slla_find_lo_or_near
#define map_xx_find_exact_hi	map_slla_find_exact_hi
#define map_xx_find_hi_or_near	map_slla_find_hi_or_near
#define map_xx_insert		map_slla_insert
#define map_xx_insert_dup	map_slla_insert_dup
#define map_xx_selected		map_slla_selected
#define map_xx_key_ptr		map_slla_key_ptr
#define map_xx_key_dup		map_slla_key_dup
#define map_xx_key_copy		map_slla_key_copy
#define map_xx_key_len		map_slla_key_len
#define map_xx_store		map_slla_store
#define map_xx_store_share	map_slla_store_share
#define map_xx_fetch_ptr	map_slla_fetch_ptr
#define map_xx_fetch_dup	map_slla_fetch_dup
#define map_xx_fetch_copy	map_slla_fetch_copy
#define map_xx_fetch_len	map_slla_fetch_len
#define map_type		signed long long

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ulla_*
//
// For descriptions of each function, see map_array.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ulla_new
#define map_xx_cmp_func		map_ulla_cmp_func
#define map_xx_find		map_ulla_find
#define map_xx_find_exact	map_ulla_find_exact
#define map_xx_find_exact_lo	map_ulla_find_exact_lo
#define map_xx_find_lo_or_near	map_ulla_find_lo_or_near
#define map_xx_find_exact_hi	map_ulla_find_exact_hi
#define map_xx_find_hi_or_near	map_ulla_find_hi_or_near
#define map_xx_insert		map_ulla_insert
#define map_xx_insert_dup	map_ulla_insert_dup
#define map_xx_selected		map_ulla_selected
#define map_xx_key_ptr		map_ulla_key_ptr
#define map_xx_key_dup		map_ulla_key_dup
#define map_xx_key_copy		map_ulla_key_copy
#define map_xx_key_len		map_ulla_key_len
#define map_xx_store		map_ulla_store
#define map_xx_store_share	map_ulla_store_share
#define map_xx_fetch_ptr	map_ulla_fetch_ptr
#define map_xx_fetch_dup	map_ulla_fetch_dup
#define map_xx_fetch_copy	map_ulla_fetch_copy
#define map_xx_fetch_len	map_ulla_fetch_len
#define map_type		unsigned long long

#include "map_array.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key_ptr
#undef map_xx_key_dup
#undef map_xx_key_copy
#undef map_xx_key_len
#undef map_xx_store
#undef map_xx_store_share
#undef map_xx_fetch_ptr
#undef map_xx_fetch_dup
#undef map_xx_fetch_copy
#undef map_xx_fetch_len
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_si_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_si_new
#define map_xx_cmp_func		map_si_cmp_func
#define map_xx_find		map_si_find
#define map_xx_find_exact	map_si_find_exact
#define map_xx_find_exact_lo	map_si_find_exact_lo
#define map_xx_find_lo_or_near	map_si_find_lo_or_near
#define map_xx_find_exact_hi	map_si_find_exact_hi
#define map_xx_find_hi_or_near	map_si_find_hi_or_near
#define map_xx_insert		map_si_insert
#define map_xx_insert_dup	map_si_insert_dup
#define map_xx_selected		map_si_selected
#define map_xx_key		map_si_key
#define map_xx_store		map_si_store
#define map_xx_fetch		map_si_fetch
#define map_type		signed int

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ui_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ui_new
#define map_xx_cmp_func		map_ui_cmp_func
#define map_xx_find		map_ui_find
#define map_xx_find_exact	map_ui_find_exact
#define map_xx_find_exact_lo	map_ui_find_exact_lo
#define map_xx_find_lo_or_near	map_ui_find_lo_or_near
#define map_xx_find_exact_hi	map_ui_find_exact_hi
#define map_xx_find_hi_or_near	map_ui_find_hi_or_near
#define map_xx_insert		map_ui_insert
#define map_xx_insert_dup	map_ui_insert_dup
#define map_xx_selected		map_ui_selected
#define map_xx_key		map_ui_key
#define map_xx_store		map_ui_store
#define map_xx_fetch		map_ui_fetch
#define map_type		unsigned int

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_sl_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_sl_new
#define map_xx_cmp_func		map_sl_cmp_func
#define map_xx_find		map_sl_find
#define map_xx_find_exact	map_sl_find_exact
#define map_xx_find_exact_lo	map_sl_find_exact_lo
#define map_xx_find_lo_or_near	map_sl_find_lo_or_near
#define map_xx_find_exact_hi	map_sl_find_exact_hi
#define map_xx_find_hi_or_near	map_sl_find_hi_or_near
#define map_xx_insert		map_sl_insert
#define map_xx_insert_dup	map_sl_insert_dup
#define map_xx_selected		map_sl_selected
#define map_xx_key		map_sl_key
#define map_xx_store		map_sl_store
#define map_xx_fetch		map_sl_fetch
#define map_type		signed long

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ul_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ul_new
#define map_xx_cmp_func		map_ul_cmp_func
#define map_xx_find		map_ul_find
#define map_xx_find_exact	map_ul_find_exact
#define map_xx_find_exact_lo	map_ul_find_exact_lo
#define map_xx_find_lo_or_near	map_ul_find_lo_or_near
#define map_xx_find_exact_hi	map_ul_find_exact_hi
#define map_xx_find_hi_or_near	map_ul_find_hi_or_near
#define map_xx_insert		map_ul_insert
#define map_xx_insert_dup	map_ul_insert_dup
#define map_xx_selected		map_ul_selected
#define map_xx_key		map_ul_key
#define map_xx_store		map_ul_store
#define map_xx_fetch		map_ul_fetch
#define map_type		unsigned long

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_sll_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_sll_new
#define map_xx_cmp_func		map_sll_cmp_func
#define map_xx_find		map_sll_find
#define map_xx_find_exact	map_sll_find_exact
#define map_xx_find_exact_lo	map_sll_find_exact_lo
#define map_xx_find_lo_or_near	map_sll_find_lo_or_near
#define map_xx_find_exact_hi	map_sll_find_exact_hi
#define map_xx_find_hi_or_near	map_sll_find_hi_or_near
#define map_xx_insert		map_sll_insert
#define map_xx_insert_dup	map_sll_insert_dup
#define map_xx_selected		map_sll_selected
#define map_xx_key		map_sll_key
#define map_xx_store		map_sll_store
#define map_xx_fetch		map_sll_fetch
#define map_type		signed long long

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ull_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ull_new
#define map_xx_cmp_func		map_ull_cmp_func
#define map_xx_find		map_ull_find
#define map_xx_find_exact	map_ull_find_exact
#define map_xx_find_exact_lo	map_ull_find_exact_lo
#define map_xx_find_lo_or_near	map_ull_find_lo_or_near
#define map_xx_find_exact_hi	map_ull_find_exact_hi
#define map_xx_find_hi_or_near	map_ull_find_hi_or_near
#define map_xx_insert		map_ull_insert
#define map_xx_insert_dup	map_ull_insert_dup
#define map_xx_selected		map_ull_selected
#define map_xx_key		map_ull_key
#define map_xx_store		map_ull_store
#define map_xx_fetch		map_ull_fetch
#define map_type		unsigned long long

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ptr_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ptr_new
#define map_xx_cmp_func		map_ptr_cmp_func
#define map_xx_find		map_ptr_find
#define map_xx_find_exact	map_ptr_find_exact
#define map_xx_find_exact_lo	map_ptr_find_exact_lo
#define map_xx_find_lo_or_near	map_ptr_find_lo_or_near
#define map_xx_find_exact_hi	map_ptr_find_exact_hi
#define map_xx_find_hi_or_near	map_ptr_find_hi_or_near
#define map_xx_insert		map_ptr_insert
#define map_xx_insert_dup	map_ptr_insert_dup
#define map_xx_selected		map_ptr_selected
#define map_xx_key		map_ptr_key
#define map_xx_store		map_ptr_store
#define map_xx_fetch		map_ptr_fetch
#define map_type		void *

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_d_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_d_new
#define map_xx_cmp_func		map_d_cmp_func
#define map_xx_find		map_d_find
#define map_xx_find_exact	map_d_find_exact
#define map_xx_find_exact_lo	map_d_find_exact_lo
#define map_xx_find_lo_or_near	map_d_find_lo_or_near
#define map_xx_find_exact_hi	map_d_find_exact_hi
#define map_xx_find_hi_or_near	map_d_find_hi_or_near
#define map_xx_insert		map_d_insert
#define map_xx_insert_dup	map_d_insert_dup
#define map_xx_selected		map_d_selected
#define map_xx_key		map_d_key
#define map_xx_store		map_d_store
#define map_xx_fetch		map_d_fetch
#define map_type		double

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// functions	map_ld_*
//
// For descriptions of each function, see map_scalar.h
//-----------------------------------------------------------------------------
#define map_xx_new		map_ld_new
#define map_xx_cmp_func		map_ld_cmp_func
#define map_xx_find		map_ld_find
#define map_xx_find_exact	map_ld_find_exact
#define map_xx_find_exact_lo	map_ld_find_exact_lo
#define map_xx_find_lo_or_near	map_ld_find_lo_or_near
#define map_xx_find_exact_hi	map_ld_find_exact_hi
#define map_xx_find_hi_or_near	map_ld_find_hi_or_near
#define map_xx_insert		map_ld_insert
#define map_xx_insert_dup	map_ld_insert_dup
#define map_xx_selected		map_ld_selected
#define map_xx_key		map_ld_key
#define map_xx_store		map_ld_store
#define map_xx_fetch		map_ld_fetch
#define map_type		long double

#include "map_scalar.h"

#undef map_xx_new
#undef map_xx_cmp_func
#undef map_xx_find
#undef map_xx_find_exact
#undef map_xx_find_exact_lo
#undef map_xx_find_lo_or_near
#undef map_xx_find_exact_hi
#undef map_xx_find_hi_or_near
#undef map_xx_insert
#undef map_xx_insert_dup
#undef map_xx_selected
#undef map_xx_key
#undef map_xx_store
#undef map_xx_fetch
#undef map_type

//-----------------------------------------------------------------------------
// function	map_key_type
//
// purpose	Return the key type of the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) >0 = key type returned
//-----------------------------------------------------------------------------
int
map_key_type (
    MAP         arg_map
);

//-----------------------------------------------------------------------------
// function	map_data_type
//
// purpose	Return the data type of the active member of a mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no active member
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_data_type (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_data_clear
// alias	map_none_store
//
// purpose	Clear the data value of the active member of a mapping so
//		that it is just like it was newly inserted.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no current member to clear
//		(int)  0 = current member cleared OK
//-----------------------------------------------------------------------------
int
map_data_clear (
    MAP         arg_map
);
#define map_none_store(m) map_data_clear((m))

//-----------------------------------------------------------------------------
// function	map_first
//
// purpose	Select the first element of a mapping as the current one.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no elements in map
//		(int)  1 = first element selected OK
//-----------------------------------------------------------------------------
int
map_first (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_last
//
// purpose	Select the last element of a mapping as the current one.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no elements in map
//		(int)  1 = last element selected OK
//-----------------------------------------------------------------------------
int
map_last (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_forward
//
// purpose	Move the current element selection of a mapping one step in
//		the forward direction.  If there is no current element, then
//		select the first element.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no elements in map
//		(int)  1 = next forward element selected OK
//-----------------------------------------------------------------------------
int
map_forward (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_reverse
//
// purpose	Move the current element selection of a mapping one step in
//		the reverse direction.  If there is no current element, then
//		select the last element.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no elements in map
//		(int)  1 = next forward element selected OK
//-----------------------------------------------------------------------------
int
map_reverse (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_unselect
//
// purpose	Clear the current element selection and make the mapping have
//		no currently selected element.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no element was selected before
//		(int)  1 = an element was selected before
//-----------------------------------------------------------------------------
int
map_unselect (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_delete
//
// purpose	Delete the currently selected element of a mapping without
//		consideration of the key type.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int)  0 = no current element to delete
//		(int)  1 = current element deleted OK
//-----------------------------------------------------------------------------
int
map_delete (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_empty
//
// purpose	Delete all the elements of a mapping without consideration of
//		the key type.
//
// arguments	1 (MAP) mapping handle
//
// returns	(long) -2 = error, no map
//		(long) number of elements deleted
//-----------------------------------------------------------------------------
long
map_empty (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_destroy
//
// purpose	Destroy a mapping.  Delete all the nodes and free the mapping.
//		The mapping handle becomes invalid and must not be used again.
//
// arguments	1 (MAP) mapping handle
//
// returns	(long) -2 = error, no map
//		(long) number of elements deleted
//
// note		In UNIX and similar virtual memory systems, if the process
//		is near the end of its execution, it is likely not necessary
//		to free the remaining space, as virtual memory cleanup will
//		remove it more efficiently.
//-----------------------------------------------------------------------------
long
map_destroy (
    MAP		arg_map
);

//-----------------------------------------------------------------------------
// function	map_str_put
//
// purpose	Put a string indexed by a string in a mapping with both
//		the key and data in the same function.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key string
//		3 (const char *) data string
//
// returns	(int) -2 = error, no map
//		(int)  0 = data inserted or replaced
//-----------------------------------------------------------------------------
int
map_str_put (
    MAP			arg_map
    ,
    const char *	arg_key_str
    ,
    const char *	arg_data_str
);

//-----------------------------------------------------------------------------
// function	map_str_get_dup
//
// purpose	Get a string indexed by a string from a mapping with both
//		the key and the data in the same function.  Return a
//		duplicate of the string that the caller must free.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key string
//		3 (const char *) default data string
//
// returns	(char *) duplicate data string or duplicate of default data
//-----------------------------------------------------------------------------
char *
map_str_get_dup (
    MAP			arg_map
    ,
    const char *	arg_key_str
    ,
    const char *	arg_default_str
);

//-----------------------------------------------------------------------------
// function	map_str_get_ptr
//
// purpose	Get a string indexed by a string from a mapping with both
//		the key and the data in the same function.  Return a
//		pointer to the string that the caller must not change.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key string
//		3 (const char *) default data string
//
// returns	(const char *) data string pointer or default data pointer
//-----------------------------------------------------------------------------
const char *
map_str_get_ptr (
    MAP			arg_map
    ,
    const char *	arg_key_str
    ,
    const char *	arg_default_str
);

//-----------------------------------------------------------------------------
// end of map.h
//-----------------------------------------------------------------------------
#endif /* __MAP_H__ */
