//-----------------------------------------------------------------------------
// Copyright  2000 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	avlmap
// homepage	http://phil.ipal.org/freeware/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map_array.h
//
// purpose	Declare functions for the key and data type specified by
//		the source file that includes this file.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// The following must be defined by the map.h header which includes this one:
//
// map_type		Typedef this to the C type for the array type to be
//			used for the key or data, e.g. long double.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// function	map_xx_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_xx_new ();

#ifdef map_xx_cmp_func
//-----------------------------------------------------------------------------
// function     map_xx_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments    1 (MAP) mapping handle
//              2 (int *)() alternate comparison function
//
// comparison   1 (const map_type *) pointer to string 1
//  arguments   2 (size_t) count of characters in string 1
//              3 (const map_type *) pointer to string 2
//              4 (size_t) count of characters in string 2
//
// comparison   (int) -1: key 1 <  key 2
//    returns   (int)  0: key 1 == key 2
//              (int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_xx_cmp_func (
    MAP         arg_map
    ,
    int         (* arg_compare)(const map_type *
                                ,
                                size_t
                                ,
                                const map_type *
                                ,
                                size_t
                                )
    );
#endif

#ifdef map_xx_cmp_options
//-----------------------------------------------------------------------------
// function     map_xx_cmp_options
//
// purpose      Set the comparison feature to enable or disable the options
//              specified in the 3rd argument
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//              3 (int) which option
//
// returns      (int)  < 0 : error
//              (int) == 0 : OK, all previously disabled
//              (int)  > 0 : OK, one or more previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_options (
    MAP         arg_map
    ,
    int         arg_state
    ,
    int         arg_flag
    );

#ifdef map_xx_cmp_lower
//-----------------------------------------------------------------------------
// function     map_xx_cmp_lower
//
// purpose      Set the comparison feature to enable or disable alphabetic
//              case insensitive comparison by means of lower case conversion.
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//
// returns      (int) -1 : error
//              (int)  0 : OK, previously disabled
//              (int)  1 : OK, previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_lower (
    MAP         arg_map
    ,
    int         arg_lower
    );
#endif /* map_xx_cmp_lower */

#ifdef map_xx_cmp_upper
//-----------------------------------------------------------------------------
// function     map_xx_cmp_upper
//
// purpose      Set the comparison feature to enable or disable alphabetic
//              case insensitive comparison by means of upper case conversion.
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//
// returns      (int) -1 : error
//              (int)  0 : OK, previously disabled
//              (int)  1 : OK, previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_upper (
    MAP         arg_map
    ,
    int         arg_upper
    );
#endif /* map_xx_cmp_upper */

#ifdef map_xx_cmp_digits
//-----------------------------------------------------------------------------
// function     map_xx_cmp_digits
//
// purpose      Set the comparison feature to enable or disable comparing
//              substrings of decimal digits by their numeric value.
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//
// returns      (int) -1 : error
//              (int)  0 : OK, previously disabled
//              (int)  1 : OK, previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_digits (
    MAP         arg_map
    ,
    int         arg_digits
    );
#endif /* map_xx_cmp_digits */

#ifdef map_xx_cmp_spaces
//-----------------------------------------------------------------------------
// function     map_xx_cmp_spaces
//
// purpose      Set the comparison feature to enable or disable comparing
//              substrings of non-graphical characters as being collapsed
//              to a single binary 0 character.
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//
// returns      (int) -1 : error
//              (int)  0 : OK, previously disabled
//              (int)  1 : OK, previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_spaces (
    MAP         arg_map
    ,
    int         arg_spaces
    );
#endif /* map_xx_cmp_spaces */

#ifdef map_xx_cmp_paths
//-----------------------------------------------------------------------------
// function     map_xx_cmp_paths
//
// purpose      Set the comparison feature to enable or disable file paths
//              with the '/' character lower than other characters.
//
// arguments    1 (MAP) mapping handle
//              2 (int) -1: no change, 0: disable, 1: enable
//
// returns      (int) -1 : error
//              (int)  0 : OK, previously disabled
//              (int)  1 : OK, previously enabled
//-----------------------------------------------------------------------------
int
map_xx_cmp_paths (
    MAP         arg_map
    ,
    int         arg_paths
    );
#endif /* map_xx_cmp_paths */

#endif /* map_xx_cmp_options */

//-----------------------------------------------------------------------------
// function	map_xx_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
    );

//-----------------------------------------------------------------------------
// function	map_xx_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find_exact (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find_exact_lo (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find_lo_or_near (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find_exact_hi (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_xx_find_hi_or_near (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_xx_insert (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_xx_insert_dup (
    MAP			arg_map
    ,
    const map_type *	arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const map_type *) pointer to shared key
//		(const map_type *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const map_type *
map_xx_key_ptr (
    MAP			arg_map
    );

//-----------------------------------------------------------------------------
// function	map_xx_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(map_type *) pointer to key duplicate copy
//		(map_type *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
map_type *
map_xx_key_dup (
    MAP			arg_map
    );

//-----------------------------------------------------------------------------
// function	map_xx_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (map_type *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0 = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_xx_key_copy (
    MAP			arg_map
    ,
    map_type *		arg_key_ptr
    ,
    size_t		arg_key_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0 = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_xx_key_len (
    MAP			arg_map
    );

//-----------------------------------------------------------------------------
// function	map_xx_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const map_type *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_xx_store (
    MAP			arg_map
    ,
    const map_type *	arg_data_ptr
    ,
    size_t		arg_data_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (map_type *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_xx_store_share (
    MAP			arg_map
    ,
    map_type *	arg_data_ptr
    ,
    size_t		arg_data_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const map_type *) pointer to data string array
//		(const map_type *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const map_type *
map_xx_fetch_ptr (
    MAP			arg_map
    );

//-----------------------------------------------------------------------------
// function	map_xx_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(map_type *) pointer to duplicate data string array
//		(map_type *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
map_type *
map_xx_fetch_dup (
    MAP			arg_map
    );

//-----------------------------------------------------------------------------
// function	map_xx_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (map_type *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0 = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_xx_fetch_copy (
    MAP			arg_map
    ,
    map_type *		arg_data_ptr
    ,
    size_t		arg_data_len
    );

//-----------------------------------------------------------------------------
// function	map_xx_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (map_type * *) pointer where to store data
//
// returns	(size_t) ~0 = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_xx_fetch_len (
    MAP			arg_map
    ,
    map_type * *	arg_data_p
    );

//-----------------------------------------------------------------------------
// End of function declarations.
//-----------------------------------------------------------------------------
